@@ -551,8 +551,13 @@ zvt_term_background_unload(ZvtTerm *term)
     zp->background = 0;
   }
 
-  /* free pixmap also ... must take into account root pixmaps */
-  /* *FIXME FIXME* */
+  /* free pixmap also ... must take into account root pixmaps (what?) */
+  if (zp->background_pixmap) {
+      if (term->back_gc)
+	  gdk_gc_set_fill (term->back_gc, GDK_SOLID);
+      g_object_unref (zp->background_pixmap);
+  }
+  
   zp->background_pixmap = 0;
   gtk_widget_queue_draw(GTK_WIDGET(term));
 }
@@ -687,7 +692,8 @@ zvt_background_set(ZvtTerm *term)
       int pwidth, pheight;
       gdk_drawable_get_size(pixmap, &pwidth, &pheight);
       pixbuf = gdk_pixbuf_get_from_drawable(0, pixmap, cmap, 0, 0, 0, 0, pwidth, pheight);
-      /* free the pixmap? */
+      /* free the pixmap */
+      g_object_unref (pixmap);
     }
 
     if (pixbuf != NULL) {
@@ -741,6 +747,10 @@ zvt_background_set(ZvtTerm *term)
     g_object_unref(pixbuf);
   }
 
+  /* unref the previous pixmap */
+  if (zp->background_pixmap)
+      g_object_unref (zp->background_pixmap);
+  
   zp->background_pixmap = pixmap;
 
   if (term->back_gc) {
