@@ -923,6 +923,9 @@ void vt_fix_selection(struct _vtx *vx)
 	sx--;
       if (!vt_in_wordclass(vx, s->data[sx]))
 	sx++;
+      /* FIXME: workaround for bug #73284 */
+      if (sx == ex && sy == ey)
+        sx--;
     }
     d(printf("%d\n", sx));
 
@@ -973,17 +976,17 @@ void vt_fix_selection(struct _vtx *vx)
 /* convert columns start to column end of line l, into
    chars of size 'size'.
 */
-static char *vt_expand_line(struct vt_line *l, int size, int start, int end,
-			    gssize *datalen)
+static char *vt_expand_line(struct vt_line *l, int size, int start, int end, char *out)
 {
   int i;
+  unsigned int c=0;
   int state=0;
   int dataend=0;
   int lf=0;
 
   /* scan from the end of the line to the start, looking for the
      actual end of screen data on that line */
-  for (dataend = l->width; dataend > 0;) {
+  for(dataend=l->width;dataend>-1;) {
     dataend--;
     if (l->data[dataend] & VTATTR_DATAMASK) {
       dataend++;
@@ -991,49 +994,14 @@ static char *vt_expand_line(struct vt_line *l, int size, int start, int end,
     }
   }
 
-  if (end > dataend) {
+  if (end>dataend) {
     lf = 1;			/* we selected past the end of the line */
     end = dataend;
   }
 
   if (start<0)
     start=0;
-
-  {
-    gssize len = end - start;
-    char *buf = g_new (char, len + 2);
-    char *bufp;
-    gsize bytes_read = 0;
-    gsize convlen = 0;
-    gchar *convbuf;
-    GError *error = NULL;
-    uint32 c;
-    for (bufp = buf, i = start; i < end; i++, bufp++) {
-      c = VT_ASCII(l->data[i]);
-      *bufp = c & 0x0000000ff;
-    }
-
-    if (lf)
-      buf[len++] = '\n';
-
-    buf[len] = 0;
-
-    convbuf = g_locale_to_utf8 (buf, len, &bytes_read, &convlen, &error);
-    g_free (buf);
-
-    if (error)
-      {
-	g_warning ("Error selecection data to UTF-8: %s\n", error->message);
-	g_error_free (error);
-	*datalen = 0;
-	return 0;
-      }
-
-    convbuf[convlen] = '\0';
-    *datalen = convlen;
-    return convbuf;
-  }
-#ifdef YUCK 
+ 
   /* YUCK!  oh well, this seems easiest way to do this! */
   switch(size) {
   case 2: {
@@ -1115,7 +1083,6 @@ static char *vt_expand_line(struct vt_line *l, int size, int start, int end,
   break;
   }
   return out;
-#endif /* YUCK */
 }
 
 /*
@@ -1133,7 +1100,7 @@ vt_select_block(struct _vtx *vx, int size, int sx, int sy, int ex, int ey, int *
 {
   struct vt_line *wn, *nn;
   int line;
-  char *data;
+  char *out, *data;
   int tmp;
 
   if ( ((sy == ey) && (sx > ex)) ||
@@ -1143,6 +1110,15 @@ vt_select_block(struct _vtx *vx, int size, int sx, int sy, int ex, int ey, int *
     tmp = sy; sy=ey; ey=tmp;
   }
 
+  /* makes a rough assumption about the buffer size needed */
+  if ( (data = g_malloc(size*(ey-sy+1)*(vx->vt.width+20)+1)) == 0 ) {
+    *len = 0;
+    printf("ERROR: Cannot g_malloc selection buffer\n");
+    return 0;
+  }
+
+  out = data;
+
   line = sy;
   wn = (struct vt_line *)vt_list_index((line<0)?(&vx->vt.scrollback):(&vx->vt.lines), line);
 
@@ -1158,15 +1134,15 @@ vt_select_block(struct _vtx *vx, int size, int sx, int sy, int ex, int ey, int *
 
   /* only a single line selected? */
   if (sy == ey) {
-    data = vt_expand_line(wn, size, sx, ex, len);
+    out = vt_expand_line(wn, size, sx, ex, out);
   } else {
     /* scan lines */
     while (nn && (line<ey)) {
       d(printf("adding selection from line %d\n", line));
       if (line == sy) {
-	data = vt_expand_line(wn, size, sx, wn->width, len); /* first line */
+	out = vt_expand_line(wn, size, sx, wn->width, out); /* first line */
       } else {
-	data = vt_expand_line(wn, size, 0, wn->width, len);
+	out = vt_expand_line(wn, size, 0, wn->width, out);
       }
       line++;
       if (line==0) {		/* wrapped into 'on screen' area? */
@@ -1180,9 +1156,12 @@ vt_select_block(struct _vtx *vx, int size, int sx, int sy, int ex, int ey, int *
 
     /* last line (if it exists - shouldn't happen?) */
     if (nn)
-      data = vt_expand_line(wn, size, 0, ex, len);
+      out = vt_expand_line(wn, size, 0, ex, out);
   }
 
+  *len = (out-data)/size;
+  *out = 0;
+
   d(printf("selected text = \n");
     fwrite(data, out-data, 1, stdout);
     printf("\n"));
