@@ -1552,16 +1552,8 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	continue;
       }
     } else {
-      if (c < 0x80)
-	{
-	  mode = modes[c & 0x7f].modes;
-	  process = modes[c & 0x7f].process;
-	}
-      else
-	{
-	  mode = VT_LIT;
-	  process = 0;
-	}
+      mode = modes[c & 0x7f].modes;
+      process = modes[c & 0x7f].process;
     }
 #else
     mode = modes[c & 0x7f].modes;
